package org.communiquons.dccaggregator.apis

import android.content.Context
import okhttp3.HttpUrl.Companion.toHttpUrl
import okhttp3.OkHttpClient
import okhttp3.tls.HandshakeCertificates
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.security.cert.CertificateFactory
import java.security.cert.X509Certificate
import java.util.concurrent.TimeUnit

@Suppress("SameParameterValue")
private fun certificateFromString(context: Context, fileName: String): X509Certificate {
    return CertificateFactory.getInstance("X.509").generateCertificate(
        context.resources.openRawResource(
            context.resources.getIdentifier(
                fileName,
                "raw", context.packageName
            )
        )
    ) as X509Certificate
}

private var defaultOkHTTPClient: OkHttpClient? = null

fun initOkHTTP(context: Context) {
    if(defaultOkHTTPClient == null)
        defaultOkHTTPClient = getDefaultOKHttpClient(context)
}

fun getDefaultOKHttpClient(context: Context): OkHttpClient {
    return OkHttpClient.Builder().apply {
        val certificates: HandshakeCertificates = HandshakeCertificates.Builder()
            .addPlatformTrustedCertificates()
            .addTrustedCertificate(certificateFromString(context, "internal_july_2017"))
            .build()
        sslSocketFactory(certificates.sslSocketFactory(), certificates.trustManager)
        callTimeout(1L, TimeUnit.MINUTES)
        connectTimeout(1L, TimeUnit.MINUTES)
        readTimeout(1L, TimeUnit.MINUTES)
        writeTimeout(1L, TimeUnit.MINUTES)
    }.build()
}

fun <T> getRetrofitService(
    baseUrl: String,
    clazz: Class<T>,
): T {
    return Retrofit.Builder()
        .baseUrl(baseUrl.toHttpUrl())
        .addConverterFactory(GsonConverterFactory.create())
        .client(defaultOkHTTPClient!!)
        .build().create(clazz)
}