package org.communiquons.dccaggregator.data

import android.content.Context
import dgca.verifier.app.decoder.CertificateDecodingResult
import dgca.verifier.app.decoder.DefaultCertificateDecoder
import dgca.verifier.app.decoder.base45.Base45Decoder
import dgca.verifier.app.decoder.model.GreenCertificate
import dgca.verifier.app.decoder.model.RecoveryStatement
import dgca.verifier.app.decoder.model.Vaccination
import org.communiquons.dccaggregator.R
import org.communiquons.dccaggregator.RESULT_TEST_POSITIVE
import org.communiquons.dccaggregator.TEST_ANTIGENIC
import org.communiquons.dccaggregator.TEST_PCR

@OptIn(ExperimentalUnsignedTypes::class)
class Certificate(val encoded: String) {

    // Decoded green certificate
    val decoded: GreenCertificate

    init {
        val res = DefaultCertificateDecoder(Base45Decoder()).decodeCertificate(encoded)

        if (res is CertificateDecodingResult.Error)
            throw Exception("Failed to read certificate!", res.error.error)

        decoded = (res as CertificateDecodingResult.Success).greenCertificate
    }

    /**
     * Get person full name
     */
    val fullName get() = "${decoded.person.standardisedGivenName} ${decoded.person.standardisedFamilyName}"

    /**
     * Get certificate id
     */
    val id get() = decoded.getDgci()

    val dateOfBirth get() = decoded.dateOfBirth

    val test get() = decoded.tests?.getOrNull(0)

    val vaccination: Vaccination? get() = decoded.vaccinations?.getOrNull(0)

    val recovery: RecoveryStatement? get() = decoded.recoveryStatements?.getOrNull(0)

    fun descText(ctx: Context): String {
        if (vaccination != null)
            return ctx.resources.getString(
                R.string.desc_vaccine_certificate,
                vaccination!!.doseNumber,
                vaccination!!.totalSeriesOfDoses,
                vaccination!!.dateOfVaccination,
                vaccination!!.vaccine
            )

        if (recovery != null)
            return ctx.resources.getString(
                R.string.desc_recovery_certificate,
                recovery!!.dateOfFirstPositiveTest
            )

        if (test != null)
            return ctx.resources.getString(
                R.string.desc_test_certificate,
                ctx.resources.getString(
                    if (test!!.testResult == RESULT_TEST_POSITIVE)
                        R.string.positive
                    else
                        R.string.negative
                ).uppercase(),
                test!!.dateTimeOfCollection,
                ctx.resources.getString(when(test!!.typeOfTest) {
                    TEST_ANTIGENIC -> R.string.antigenic
                    TEST_PCR -> R.string.pcr
                    else -> R.string.unkown_test_type
                })
            )

        return ctx.resources.getString(R.string.desc_unknown_certificate)
    }
}